@@ -164,8 +164,8 @@ update_sources_view (GdictPrefDialog *dialog)
       gtk_list_store_append (dialog->sources_list, &iter);
       gtk_list_store_set (dialog->sources_list, &iter,
       			  SOURCES_ACTIVE_COLUMN, is_active,
-      			  SOURCES_NAME_COLUMN, g_strdup (name),
-      			  SOURCES_DESCRIPTION_COLUMN, g_strdup (description),
+      			  SOURCES_NAME_COLUMN, name,
+      			  SOURCES_DESCRIPTION_COLUMN, description,
       			  -1);
     }
 
@@ -667,12 +667,10 @@ gdict_pref_dialog_init (GdictPrefDialog *dialog)
   dialog->sources_view = glade_xml_get_widget (dialog->xml, "sources_treeview");
   build_sources_view (dialog);
 
-  dialog->active_source = gconf_client_get_string (dialog->gconf_client,
-		  				   GDICT_GCONF_SOURCE_KEY,
-						   NULL);
-  if (!dialog->active_source)
-    dialog->active_source = g_strdup (GDICT_DEFAULT_SOURCE_NAME);
-  
+  dialog->active_source = gdict_gconf_get_string_with_default (dialog->gconf_client,
+							       GDICT_GCONF_SOURCE_KEY,
+							       GDICT_DEFAULT_SOURCE_NAME);
+
   dialog->sources_add = glade_xml_get_widget (dialog->xml, "add_button");
   gtk_tooltips_set_tip (dialog->tips,
   			dialog->sources_add,
@@ -717,13 +715,13 @@ gdict_pref_dialog_init (GdictPrefDialog *dialog)
 }
 
 void
-gdict_show_pref_dialog (GtkWindow         *parent,
+gdict_show_pref_dialog (GtkWidget         *parent,
 			const gchar       *title,
 			GdictSourceLoader *loader)
 {
   GtkWidget *dialog;
   
-  g_return_if_fail ((parent == NULL) || GTK_IS_WINDOW (parent));
+  g_return_if_fail (GTK_IS_WIDGET (parent));
   g_return_if_fail (GDICT_IS_SOURCE_LOADER (loader));
   
   if (parent)
@@ -744,9 +742,9 @@ gdict_show_pref_dialog (GtkWindow         *parent,
                         G_CALLBACK (gtk_widget_hide_on_delete),
                         NULL);
       
-      if (parent)
+      if (parent && GTK_IS_WINDOW (parent))
         {
-          gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
+          gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
           gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
           g_object_set_data_full (G_OBJECT (parent), "gdict-pref-dialog",
                                   dialog,
@@ -755,6 +753,8 @@ gdict_show_pref_dialog (GtkWindow         *parent,
       else
         global_dialog = dialog;
     }
-  
-  gtk_window_present (GTK_WINDOW (dialog));
+
+  gtk_window_set_screen (GTK_WINDOW (dialog),
+ 			 gtk_widget_get_screen (parent));
+  gtk_window_present (GTK_WINDOW (dialog));  
 }
