@@ -55,6 +55,11 @@
 #define GDICT_WINDOW_MIN_WIDTH	  400
 #define GDICT_WINDOW_MIN_HEIGHT	  330
 
+/* sidebar pages logical ids */
+#define GDICT_SIDEBAR_SPELLER_PAGE      "speller"
+#define GDICT_SIDEBAR_DATABASES_PAGE    "db-chooser"
+#define GDICT_SIDEBAR_STRATEGIES_PAGE   "strat-chooser"
+
 enum
 {
   COMPLETION_TEXT_COLUMN,
@@ -99,45 +104,9 @@ G_DEFINE_TYPE (GdictWindow, gdict_window, GTK_TYPE_WINDOW);
 
 
 static void
-gdict_window_finalize (GObject *object)
+gdict_window_finalize (GObject *gobject)
 {
-  GdictWindow *window = GDICT_WINDOW (object);
-
-  if (window->notify_id)
-    gconf_client_notify_remove (window->gconf_client, window->notify_id);
-
-  if (window->font_notify_id)
-    gconf_client_notify_remove (window->gconf_client, window->font_notify_id);
-  
-  if (window->gconf_client)
-    g_object_unref (window->gconf_client);
-  
-  if (window->context)
-    {
-      g_signal_handler_disconnect (window->context, window->lookup_start_id);
-      g_signal_handler_disconnect (window->context, window->definition_id);
-      g_signal_handler_disconnect (window->context, window->lookup_end_id);
-      g_signal_handler_disconnect (window->context, window->error_id);
-
-      g_object_unref (window->context);
-    }
-  
-  if (window->loader)
-    g_object_unref (window->loader);
-  
-  if (window->ui_manager)
-    g_object_unref (window->ui_manager);
-  
-  g_object_unref (window->action_group);
-
-  if (window->completion)
-    g_object_unref (window->completion);
-
-  if (window->completion_model)
-    g_object_unref (window->completion_model);
-
-  if (window->busy_cursor)
-    gdk_cursor_unref (window->busy_cursor);
+  GdictWindow *window = GDICT_WINDOW (gobject);
 
   g_free (window->source_name);
   g_free (window->print_font);
@@ -146,7 +115,93 @@ gdict_window_finalize (GObject *object)
   g_free (window->database);
   g_free (window->strategy);
   
-  G_OBJECT_CLASS (gdict_window_parent_class)->finalize (object);
+  G_OBJECT_CLASS (gdict_window_parent_class)->finalize (gobject);
+}
+
+static void
+gdict_window_dispose (GObject *gobject)
+{
+  GdictWindow *window = GDICT_WINDOW (gobject);
+
+  if (window->notify_id)
+    {
+      gconf_client_notify_remove (window->gconf_client, window->notify_id);
+      window->notify_id = 0;
+    }
+
+  if (window->font_notify_id)
+    {
+      gconf_client_notify_remove (window->gconf_client,
+                                  window->font_notify_id);
+      window->font_notify_id = 0;
+    }
+  
+  if (window->gconf_client)
+    {
+      g_object_unref (window->gconf_client);
+      window->gconf_client = NULL;
+    }
+  
+  if (window->context)
+    {
+      if (window->lookup_start_id)
+        {
+          g_signal_handler_disconnect (window->context,
+                                       window->lookup_start_id);
+          g_signal_handler_disconnect (window->context,
+                                       window->definition_id);
+          g_signal_handler_disconnect (window->context,
+                                       window->lookup_end_id);
+          g_signal_handler_disconnect (window->context,
+                                       window->error_id);
+
+          window->lookup_start_id = 0;
+          window->definition_id = 0;
+          window->lookup_end_id = 0;
+          window->error_id = 0;
+        }
+
+      g_object_unref (window->context);
+      window->context = NULL;
+    }
+  
+  if (window->loader)
+    {
+      g_object_unref (window->loader);
+      window->loader = NULL;
+    }
+  
+  if (window->ui_manager)
+    {
+      g_object_unref (window->ui_manager);
+      window->ui_manager = NULL;
+    }
+  
+  if (window->action_group)
+    {
+      g_object_unref (window->action_group);
+      window->action_group = NULL;
+    }
+
+  if (window->completion)
+    {
+      g_object_unref (window->completion);
+      window->completion = NULL;
+    }
+
+  if (window->completion_model)
+    {
+      g_object_unref (window->completion_model);
+      window->completion_model = NULL;
+    }
+
+  if (window->busy_cursor)
+    {
+      gdk_cursor_unref (window->busy_cursor);
+      window->busy_cursor = NULL;
+    }
+
+  G_OBJECT_CLASS (gdict_window_parent_class)->dispose (gobject);
 }
 
 static void
@@ -157,11 +212,17 @@ gdict_window_set_sidebar_visible (GdictWindow *window,
   
   if (is_visible != window->sidebar_visible)
     {
+      GtkAction *action;
+
       if (is_visible)
 	gtk_widget_show (window->sidebar_frame);
       else
 	gtk_widget_hide (window->sidebar_frame);
 
+      action = gtk_action_group_get_action (window->action_group,
+                                            "ViewSidebar");
+      gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), is_visible);
+
       window->sidebar_visible = is_visible;
     }
 }
@@ -174,11 +235,17 @@ gdict_window_set_statusbar_visible (GdictWindow *window,
 
   if (is_visible != window->statusbar_visible)
     {
+      GtkAction *action;
+
       if (is_visible)
 	gtk_widget_show (window->status);
       else
 	gtk_widget_hide (window->status);
 
+      action = gtk_action_group_get_action (window->action_group,
+                                            "ViewStatusbar");
+      gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), is_visible);
+
       window->statusbar_visible = is_visible;
     }
 }
@@ -291,7 +358,8 @@ gdict_window_error_cb (GdictContext *context,
       GdictContext *context;
 
       gdict_window_set_sidebar_visible (window, TRUE);
-      gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar), "speller");
+      gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar),
+                               GDICT_SIDEBAR_SPELLER_PAGE);
 
       /* we clone the context, so that the signals that it
        * fires do not get caught by the signal handlers we
@@ -318,21 +386,18 @@ static void
 gdict_window_set_database (GdictWindow *window,
 			   const gchar *database)
 {
-  if (window->database)
-    g_free (window->database);
+  g_free (window->database);
 
-  if (!database)
-    database = gconf_client_get_string (window->gconf_client,
-		    			GDICT_GCONF_DATABASE_KEY,
-					NULL);
-  
-  if (!database)
-    database = GDICT_DEFAULT_DATABASE;
-
-  window->database = g_strdup (database);
+  if (database)
+    window->database = g_strdup (database);
+  else
+    window->database = gdict_gconf_get_string_with_default (window->gconf_client,
+							    GDICT_GCONF_DATABASE_KEY,
+							    GDICT_DEFAULT_DATABASE);
 
   if (window->defbox)
-    gdict_defbox_set_database (GDICT_DEFBOX (window->defbox), database);
+    gdict_defbox_set_database (GDICT_DEFBOX (window->defbox),
+			       window->database);
 }
 
 static void
@@ -356,6 +421,16 @@ get_context_from_loader (GdictWindow *window)
 
   source = gdict_source_loader_get_source (window->loader,
 		  			   window->source_name);
+  if (!source &&
+      strcmp (window->source_name, GDICT_DEFAULT_SOURCE_NAME) != 0)
+    {
+      g_free (window->source_name);
+      window->source_name = g_strdup (GDICT_DEFAULT_SOURCE_NAME);
+
+      source = gdict_source_loader_get_source (window->loader,
+                                               window->source_name);
+    }
+  
   if (!source)
     {
       gchar *detail;
@@ -402,42 +477,32 @@ static void
 gdict_window_set_defbox_font (GdictWindow *window,
 			      const gchar *defbox_font)
 {
-  if (!defbox_font)
-    defbox_font = gconf_client_get_string (window->gconf_client,
-		    			   DOCUMENT_FONT_KEY,
-					   NULL);
+  g_free (window->defbox_font);
 
-  if (!defbox_font)
-    defbox_font = GDICT_DEFAULT_DEFBOX_FONT;
+  if (defbox_font)
+    window->defbox_font = g_strdup (defbox_font);
+  else
+    window->defbox_font = gdict_gconf_get_string_with_default (window->gconf_client,
+							       DOCUMENT_FONT_KEY,
+							       GDICT_DEFAULT_DEFBOX_FONT);
 
-  if (window->defbox_font)
-    g_free (window->defbox_font);
-
-  window->defbox_font = g_strdup (defbox_font);
-
-  if (!window->defbox)
-    return;
-
-  gdict_defbox_set_font_name (GDICT_DEFBOX (window->defbox),
-		  	      window->defbox_font);
+  if (window->defbox)
+    gdict_defbox_set_font_name (GDICT_DEFBOX (window->defbox),
+				window->defbox_font);
 }
 
 static void
 gdict_window_set_print_font (GdictWindow *window,
 			     const gchar *print_font)
 {
-  if (!print_font)
-    print_font = gconf_client_get_string (window->gconf_client,
-    					  GDICT_GCONF_PRINT_FONT_KEY,
-    					  NULL);
-  
-  if (!print_font)
-    print_font = GDICT_DEFAULT_PRINT_FONT;
-  
-  if (window->print_font)
-    g_free (window->print_font);
-  
-  window->print_font = g_strdup (print_font);
+  g_free (window->print_font);
+
+  if (print_font)
+    window->print_font = g_strdup (print_font);
+  else
+    window->print_font = gdict_gconf_get_string_with_default (window->gconf_client,
+							      GDICT_GCONF_PRINT_FONT_KEY,
+							      GDICT_DEFAULT_PRINT_FONT);
 }
 
 static void
@@ -521,23 +586,17 @@ gdict_window_set_source_name (GdictWindow *window,
 			      const gchar *source_name)
 {
   GdictContext *context;
-  
-  /* some sanity checks first */
-  if (!source_name)
-    source_name = gconf_client_get_string (window->gconf_client,
-      					   GDICT_GCONF_SOURCE_KEY,
-      					   NULL);
-  
-  if (!source_name)
-    source_name = GDICT_DEFAULT_SOURCE_NAME;
-  
-  if (window->source_name)
-    g_free (window->source_name);
-  
-  window->source_name = g_strdup (source_name);
-  
+
+  g_free (window->source_name);
+
+  if (source_name)
+    window->source_name = g_strdup (source_name);
+  else
+    window->source_name = gdict_gconf_get_string_with_default (window->gconf_client,
+							       GDICT_GCONF_SOURCE_KEY,
+							       GDICT_DEFAULT_SOURCE_NAME);
+
   context = get_context_from_loader (window);
-  
   gdict_window_set_context (window, context);
 }
 
@@ -620,6 +679,7 @@ gdict_window_cmd_file_new (GtkAction   *action,
 			   GdictWindow *window)
 {
   GtkWidget *new_window;
+  gchar *word = NULL;
   
   /* store the default size of the window and its state, so that
    * it's picked up by the newly created window
@@ -652,10 +712,20 @@ gdict_window_cmd_file_new (GtkAction   *action,
 		  	 GDICT_GCONF_STATUSBAR_VISIBLE_KEY,
 			 window->statusbar_visible,
 			 NULL);
- 
-  new_window = gdict_window_new (GDICT_WINDOW_ACTION_CLEAR,
-  				 window->loader,
-				 NULL, NULL);
+
+  word = gdict_defbox_get_selected_word (GDICT_DEFBOX (window->defbox));
+  if (word)
+    {
+      new_window = gdict_window_new (GDICT_WINDOW_ACTION_LOOKUP,
+                                     window->loader,
+                                     NULL, word);
+      g_free (word);
+    }
+  else
+    new_window = gdict_window_new (GDICT_WINDOW_ACTION_CLEAR,
+                                   window->loader,
+                                   NULL, NULL);
+
   gtk_widget_show (new_window);
   
   g_signal_emit (window, gdict_window_signals[CREATED], 0, new_window);
@@ -823,7 +893,7 @@ gdict_window_cmd_edit_preferences (GtkAction   *action,
 {
   g_assert (GDICT_IS_WINDOW (window));
   
-  gdict_show_pref_dialog (GTK_WINDOW (window),
+  gdict_show_pref_dialog (GTK_WIDGET (window),
   			  _("Dictionary Preferences"),
   			  window->loader);
 }
@@ -860,7 +930,8 @@ gdict_window_cmd_view_speller (GtkAction   *action,
 {
   g_assert (GDICT_IS_WINDOW (window));
 
-  gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar), "speller");
+  gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar),
+                           GDICT_SIDEBAR_SPELLER_PAGE);
   gdict_window_set_sidebar_visible (window, TRUE);
 }
 
@@ -870,7 +941,19 @@ gdict_window_cmd_view_databases (GtkAction   *action,
 {
   g_assert (GDICT_IS_WINDOW (window));
 
-  gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar), "databases");
+  gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar),
+                           GDICT_SIDEBAR_DATABASES_PAGE);
+  gdict_window_set_sidebar_visible (window, TRUE);
+}
+
+static void
+gdict_window_cmd_view_strategies (GtkAction   *action,
+                                  GdictWindow *window)
+{
+  g_assert (GDICT_IS_WINDOW (window));
+
+  gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar),
+                           GDICT_SIDEBAR_STRATEGIES_PAGE);
   gdict_window_set_sidebar_visible (window, TRUE);
 }
 
@@ -1012,16 +1095,6 @@ static const GtkActionEntry entries[] =
   { "EditPreferences", GTK_STOCK_PREFERENCES, N_("_Preferences"), NULL, NULL,
     G_CALLBACK (gdict_window_cmd_edit_preferences) },
 
-  /* View menu */
-  { "ViewSidebar", NULL, N_("_Sidebar"), "F9", NULL,
-    G_CALLBACK (gdict_window_cmd_view_sidebar) },
-  { "ViewStatusbar", NULL, N_("S_tatusbar"), NULL, NULL,
-    G_CALLBACK (gdict_window_cmd_view_statusbar) },
-  { "ViewSpeller", NULL, N_("Similar _Words"), "<control>T", NULL,
-    G_CALLBACK (gdict_window_cmd_view_speller) },
-  { "ViewDB", NULL, N_("Available _Databases"), "<control>B", NULL,
-    G_CALLBACK (gdict_window_cmd_view_databases) },
-
   /* Go menu */
   { "GoPreviousDef", GTK_STOCK_GO_BACK, N_("_Previous Definition"), "<control>Page_Up",
     N_("Go to the previous definition"), G_CALLBACK (gdict_window_cmd_go_previous_def) },
@@ -1032,6 +1105,14 @@ static const GtkActionEntry entries[] =
   { "GoLastDef", GTK_STOCK_GOTO_LAST, N_("_Last Definition"), "<control>End",
     N_("Go to the last definition"), G_CALLBACK (gdict_window_cmd_go_last_def) },
 
+  /* View menu */
+  { "ViewSpeller", NULL, N_("Similar _Words"), "<control>T", NULL,
+    G_CALLBACK (gdict_window_cmd_view_speller), },
+  { "ViewDB", NULL, N_("Available _Databases"), "<control>B", NULL,
+    G_CALLBACK (gdict_window_cmd_view_databases), },
+  { "ViewStrat", NULL, N_("Available St_rategies"), "<control>R", NULL,
+    G_CALLBACK (gdict_window_cmd_view_strategies), },
+
   /* Help menu */
   { "HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1", NULL,
     G_CALLBACK (gdict_window_cmd_help_contents) },
@@ -1044,6 +1125,14 @@ static const GtkActionEntry entries[] =
   { "Slash", GTK_STOCK_FIND, NULL, "slash", NULL, G_CALLBACK (gdict_window_cmd_edit_find) },
 };
 
+static const GtkToggleActionEntry toggle_entries[] = {
+  /* View menu */
+  { "ViewSidebar", NULL, N_("_Sidebar"), "F9", NULL,
+    G_CALLBACK (gdict_window_cmd_view_sidebar), FALSE },
+  { "ViewStatusbar", NULL, N_("S_tatusbar"), NULL, NULL,
+    G_CALLBACK (gdict_window_cmd_view_statusbar), FALSE },
+};
+
 static gboolean
 gdict_window_delete_event_cb (GtkWidget *widget,
 			      GdkEvent  *event,
@@ -1126,24 +1215,41 @@ gdict_window_gconf_notify_cb (GConfClient *client,
 }
 
 static void
-entry_activate_cb (GtkWidget   *widget,
-		   GdictWindow *window)
+lookup_word (GdictWindow *window,
+             gpointer     dummy)
 {
   const gchar *word;
-  gchar *title;
   
   g_assert (GDICT_IS_WINDOW (window));
   
   if (!window->context)
     return;
   
-  word = gtk_entry_get_text (GTK_ENTRY (widget));
+  word = gtk_entry_get_text (GTK_ENTRY (window->entry));
   if (!word || *word == '\0')
     return;
 
   gdict_window_set_word (window, word, NULL);
 }
 
+static void
+strategy_activated_cb (GdictStrategyChooser *chooser,
+                       const gchar          *strat_name,
+                       const gchar          *strat_desc,
+                       GdictWindow          *window)
+{
+  gdict_window_set_strategy (window, strat_name);
+
+  if (window->status)
+    {
+      gchar *message;
+
+      message = g_strdup_printf (_("Strategy `%s' selected"), strat_desc);
+      gtk_statusbar_push (GTK_STATUSBAR (window->status), 0, message);
+      g_free (message);
+    }
+}
+
 static void
 database_activated_cb (GdictDatabaseChooser *chooser,
 		       const gchar          *db_name,
@@ -1193,11 +1299,30 @@ sidebar_page_changed_cb (GdictSidebar *sidebar,
 
   switch (page_id[0])
     {
-    case 's': /* speller */
-      message = _("Double-click on the word to look up");
+    case 's':
+      {
+      switch (page_id[1])
+        {
+        case 'p': /* speller */
+          message = _("Double-click on the word to look up");
+          if (window->word)
+            gdict_speller_match (GDICT_SPELLER (window->speller),
+                                 window->word);
+          break;
+        case 't': /* strat-chooser */
+          message = _("Double-click on the matching strategy to use");
+          
+          gdict_strategy_chooser_refresh (GDICT_STRATEGY_CHOOSER (window->strat_chooser));
+          break;
+        default:
+          message = NULL;
+        }
+      }
       break;
     case 'd': /* db-chooser */
       message = _("Double-click on the database to use");
+      
+      gdict_database_chooser_refresh (GDICT_DATABASE_CHOOSER (window->db_chooser));
       break;
     default:
       message = NULL;
@@ -1216,6 +1341,53 @@ sidebar_closed_cb (GdictSidebar *sidebar,
   window->sidebar_visible = FALSE;
 }
 
+static void
+gdict_window_link_clicked (GdictDefbox *defbox,
+                           const gchar *link_text,
+                           GdictWindow *window)
+{
+  GtkWidget *new_window;
+  
+  /* store the default size of the window and its state, so that
+   * it's picked up by the newly created window
+   */
+  gconf_client_set_int (window->gconf_client,
+		        GDICT_GCONF_WINDOW_WIDTH_KEY,
+		  	window->default_width,
+			NULL);
+  gconf_client_set_int (window->gconf_client,
+		  	GDICT_GCONF_WINDOW_HEIGHT_KEY,
+			window->default_height,
+			NULL);
+  gconf_client_set_bool (window->gconf_client,
+		  	 GDICT_GCONF_WINDOW_IS_MAXIMIZED_KEY,
+			 window->is_maximized,
+			 NULL);
+  gconf_client_set_bool (window->gconf_client,
+		  	 GDICT_GCONF_SIDEBAR_VISIBLE_KEY,
+			 window->sidebar_visible,
+			 NULL);
+  gconf_client_set_int (window->gconf_client,
+		  	GDICT_GCONF_SIDEBAR_WIDTH_KEY,
+			window->sidebar_width,
+			NULL);
+  gconf_client_set_string (window->gconf_client,
+		  	   GDICT_GCONF_SIDEBAR_PAGE_KEY,
+			   gdict_sidebar_current_page (GDICT_SIDEBAR (window->sidebar)),
+			   NULL);
+  gconf_client_set_bool (window->gconf_client,
+		  	 GDICT_GCONF_STATUSBAR_VISIBLE_KEY,
+			 window->statusbar_visible,
+			 NULL);
+
+  new_window = gdict_window_new (GDICT_WINDOW_ACTION_LOOKUP,
+                                 window->loader,
+                                 NULL, link_text);
+  gtk_widget_show (new_window);
+  
+  g_signal_emit (window, gdict_window_signals[CREATED], 0, new_window);
+}
+
 static void
 gdict_window_drag_data_received_cb (GtkWidget        *widget,
 				    GdkDragContext   *context,
@@ -1372,7 +1544,7 @@ gdict_window_constructor (GType                  type,
   GtkWidget *handle;
   GtkWidget *frame1, *frame2;
   GtkWidget *vbox;
-  GtkWidget *label;
+  GtkWidget *button;
   GtkActionGroup *action_group;
   GtkAccelGroup *accel_group;
   GtkAction *action;
@@ -1400,6 +1572,9 @@ gdict_window_constructor (GType                  type,
   gtk_action_group_add_actions (action_group, entries,
   				G_N_ELEMENTS (entries),
   				window);
+  gtk_action_group_add_toggle_actions (action_group, toggle_entries,
+                                       G_N_ELEMENTS (toggle_entries),
+                                       window);
   
   window->ui_manager = gtk_ui_manager_new ();
   gtk_ui_manager_insert_action_group (window->ui_manager, action_group, 0);
@@ -1432,9 +1607,13 @@ gdict_window_constructor (GType                  type,
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
   
-  label = gtk_label_new_with_mnemonic (_("Look _up:"));
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
+  button = gtk_button_new_with_mnemonic (_("Look _up:"));
+  g_signal_connect_swapped (button, "clicked",
+                            G_CALLBACK (lookup_word),
+                            window);
+  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
 
   window->completion_model = gtk_list_store_new (COMPLETION_N_COLUMNS,
 		  				 G_TYPE_STRING);
@@ -1452,12 +1631,12 @@ gdict_window_constructor (GType                  type,
   
   gtk_entry_set_completion (GTK_ENTRY (window->entry),
 		  	    window->completion);
-  g_signal_connect (window->entry, "activate", G_CALLBACK (entry_activate_cb), window);
+  g_signal_connect_swapped (window->entry, "activate",
+                            G_CALLBACK (lookup_word),
+                            window);
   gtk_box_pack_start (GTK_BOX (hbox), window->entry, TRUE, TRUE, 0);
   gtk_widget_show (window->entry);
 
-  gtk_label_set_mnemonic_widget (GTK_LABEL (label), window->entry);
-
   handle = gtk_hpaned_new ();
   gtk_box_pack_start (GTK_BOX (vbox), handle, TRUE, TRUE, 0);
   gtk_widget_show (handle);
@@ -1469,6 +1648,10 @@ gdict_window_constructor (GType                  type,
   if (window->context)
     gdict_defbox_set_context (GDICT_DEFBOX (window->defbox), window->context);
 
+  g_signal_connect (window->defbox, "link-clicked",
+                    G_CALLBACK (gdict_window_link_clicked),
+                    window);
+
   gtk_drag_dest_set (window->defbox,
   		     GTK_DEST_DEFAULT_ALL,
   		     drop_types, n_drop_types,
@@ -1496,7 +1679,7 @@ gdict_window_constructor (GType                  type,
 		    window);
   
   gdict_sidebar_add_page (GDICT_SIDEBAR (window->sidebar),
-		          "speller",
+		          GDICT_SIDEBAR_SPELLER_PAGE,
 			  _("Similar words"),
 			  window->speller);
   gtk_widget_show (window->speller);
@@ -1509,11 +1692,24 @@ gdict_window_constructor (GType                  type,
 	  	    G_CALLBACK (database_activated_cb),
 		    window);
   gdict_sidebar_add_page (GDICT_SIDEBAR (window->sidebar),
-	  		  "db-chooser",
+	  		  GDICT_SIDEBAR_DATABASES_PAGE,
 			  _("Available dictionaries"),
 			  window->db_chooser);
   gtk_widget_show (window->db_chooser);
-  
+
+  window->strat_chooser = gdict_strategy_chooser_new ();
+  if (window->context)
+    gdict_strategy_chooser_set_context (GDICT_STRATEGY_CHOOSER (window->strat_chooser),
+                                        window->context);
+  g_signal_connect (window->strat_chooser, "strategy-activated",
+                    G_CALLBACK (strategy_activated_cb),
+                    window);
+  gdict_sidebar_add_page (GDICT_SIDEBAR (window->sidebar),
+                          GDICT_SIDEBAR_STRATEGIES_PAGE,
+                          _("Available strategies"),
+                          window->strat_chooser);
+  gtk_widget_show (window->strat_chooser);
+
   gtk_container_add (GTK_CONTAINER (frame2), window->sidebar);
   gtk_widget_show (window->sidebar);
 
@@ -1537,14 +1733,13 @@ gdict_window_constructor (GType                  type,
   gtk_box_pack_end (GTK_BOX (window->status), window->progress, FALSE, FALSE, 0);
 
   /* retrieve the font size from gconf */
-  font_name = gconf_client_get_string (window->gconf_client,
-		  		       DOCUMENT_FONT_KEY,
-				       NULL);
-  if (!font_name)
-    font_name = GDICT_DEFAULT_DEFBOX_FONT;
-  
+  font_name = gdict_gconf_get_string_with_default (window->gconf_client,
+						   DOCUMENT_FONT_KEY,
+						   GDICT_DEFAULT_DEFBOX_FONT);
+
   gdict_window_set_defbox_font (window, font_name);
   font_desc = pango_font_description_from_string (font_name);
+  g_free (font_name);
 
   sidebar_visible = gconf_client_get_bool (window->gconf_client,
 		  			   GDICT_GCONF_SIDEBAR_VISIBLE_KEY,
@@ -1602,7 +1797,9 @@ gdict_window_constructor (GType                  type,
   gtk_paned_set_position (GTK_PANED (handle),
 		          GTK_WIDGET (window)->allocation.width - sidebar_width);
   gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar), sidebar_page);
-  
+
+  g_free (sidebar_page);
+
   g_signal_connect (window, "delete-event",
 		    G_CALLBACK (gdict_window_delete_event_cb),
 		    NULL);
@@ -1613,6 +1810,8 @@ gdict_window_constructor (GType                  type,
 		    G_CALLBACK (gdict_window_handle_notify_position_cb),
 		    window);
 
+  gtk_widget_grab_focus (window->entry);
+
   gtk_widget_pop_composite_child ();
   
   return object;
@@ -1625,6 +1824,7 @@ gdict_window_class_init (GdictWindowClass *klass)
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
   gobject_class->finalize = gdict_window_finalize;
+  gobject_class->dispose = gdict_window_dispose;
   gobject_class->set_property = gdict_window_set_property;
   gobject_class->get_property = gdict_window_get_property;
   gobject_class->constructor = gdict_window_constructor;
@@ -1639,7 +1839,7 @@ gdict_window_class_init (GdictWindowClass *klass)
 						      "The default action performed by the window",
 						      GDICT_TYPE_WINDOW_ACTION,
 						      GDICT_WINDOW_ACTION_CLEAR,
-						      (G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY)));
+						      (G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY)));
   g_object_class_install_property (gobject_class,
   				   PROP_SOURCE_LOADER,
   				   g_param_spec_object ("source-loader",
@@ -1768,6 +1968,7 @@ gdict_window_new (GdictWindowAction  action,
 		  const gchar       *word)
 {
   GtkWidget *retval;
+  GdictWindow *window;
   
   g_return_val_if_fail (GDICT_IS_SOURCE_LOADER (loader), NULL);
   
@@ -1777,18 +1978,47 @@ gdict_window_new (GdictWindowAction  action,
 			 "source-name", source_name,
 			 NULL);
 
+  window = GDICT_WINDOW (retval);
+
   if (word && word[0] != '\0')
     {
       switch (action)
         {
 	case GDICT_WINDOW_ACTION_LOOKUP:
-	  gtk_entry_set_text (GTK_ENTRY (GDICT_WINDOW (retval)->entry), word);
-	  gdict_window_set_word (GDICT_WINDOW (retval), word, NULL);
+	  gtk_entry_set_text (GTK_ENTRY (window->entry), word);
+	  gdict_window_set_word (window, word, NULL);
 	  break;
 	case GDICT_WINDOW_ACTION_MATCH:
-	  gtk_entry_set_text (GTK_ENTRY (GDICT_WINDOW (retval)->entry), word);
-	  /* FIXME */
+          {
+          GdictSource *source;
+          GdictContext *context;
+
+	  gtk_entry_set_text (GTK_ENTRY (window->entry), word);
+          
+          gdict_window_set_sidebar_visible (window, TRUE);
+          gdict_sidebar_view_page (GDICT_SIDEBAR (window->sidebar),
+                                   GDICT_SIDEBAR_SPELLER_PAGE);
+
+          /* we clone the context, so that the signals that it
+           * fires do not get caught by the signal handlers we
+           * use for getting the definitions.
+           */
+          source = gdict_source_loader_get_source (window->loader,
+                                                   window->source_name);
+          context = gdict_source_get_context (source);
+
+          gdict_speller_set_context (GDICT_SPELLER (window->speller), context);
+          
+          g_object_unref (context);
+          g_object_unref (source);
+      
+          gdict_speller_set_strategy (GDICT_SPELLER (window->speller),
+                                      window->strategy);
+      
+          gdict_speller_match (GDICT_SPELLER (window->speller), word);
+          }
 	case GDICT_WINDOW_ACTION_CLEAR:
+          gdict_defbox_clear (GDICT_DEFBOX (window->defbox));
 	  break;
 	default:
 	  g_assert_not_reached ();
